/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

var IX_KIND    = 0;
var IX_FORMAT  = 1;
var IX_AUDIO   = 2;
var IX_VIDEO   = 3;
var IX_PROFILE = 4;

// defined in dconf.html

if (!xcfg) { xcfg = {} }
if (!DDEF) { DDEF = {} }

//-----------------------------------------------------------------------------

var reso = {
  '16:9': {
    '854x480': 'NTSC',
    '1024x576': 'PAL',
    '1280x720': 'HD 720p',
    '1920x1080': 'HD 1080p',
    '3840x2160': '4K UHDTV',
    '7680x4320': '8K UHDTV'
  },
  '4:3': {
    '640x480': 'NTSC',
    '768x576': 'PAL',
    '1024x768': 'XGA',
    '1600x1200': 'UXGA'
  },
  'iPhone': {
    '480x320': 'iPhone 3',
    '960x640': 'iPhone 4',
    '1136x640': 'iPhone 5',
    '1334x750': 'iPhone 6',
    '2208x1242': 'iPhone 6 Plus'
  },
  'iPad': {
    '2048x1536': 'iPad Retina'
  },
  'Other': {
    'cst': 'Custom'
  }
};

var formats = {
  Audio: {
    'MP-3': ['a', 'mpeg audio'],
    'MP-4': ['a', 'm4a'],
    'AAC':  ['a', 'adts'],
    'FLAC': ['a', 'flac'],
    'WAV':  ['a', 'wave'],
    'WMA':  ['a', 'windows media']
  },
  Video: {
    'AVI':     ["v","avi"],
    'MKV':     ["v","matroska"],
    'MP4':     ["v", "mpeg-4"],
    'MOV':     ["v", "qt"],
    'MPEG-PS': ["v","mpeg-ps"],
    'MPEG-TS': ["v","mpeg-ts"],
    'BLU-RAY': ["v", "bdav"],
    'DIVX':    ["v","divx"],
    'FLASH':   ["v","flash video"]
  }
};

//-----------------------------------------------------------------------------

function onSubmit(form, result) {

  if (!result) {
    return false;
  }

  var cfg = readData();

  if (!cfg) return;

  $('#save').disable();

  $.ajax({
    type: 'POST',
    url: about.rest + '/ssdp/dsave?uuid=' + uuid,
    contentType: 'application/json',
    data: JSON.stringify(cfg),
    processData: false
  })
    .done(function(data) {
      is.showInfo('Device configuration saved', 2000);
    })
    .fail(function(xhr) {
      is.showError(is.xhrerror(xhr), 5000);
    })
    .always(function() {
      $('#save').enable();
    });

  return false;

}

function initReso() {
  var sel = $('#sreso');
  for (var gname in reso) {
    var grp = $('<optgroup></optgroup>');
    grp.attr('label', gname);
    var kv = reso[gname];
    for (var k in kv) {
      var opt = $('<option></option>');
      opt.attr('value', k);
      var text = kv[k];
      if (k !== 'cst') {
        text = k + ' - ' + text;
      }
      opt.text(text);
      grp.append(opt);
    }
    sel.append(grp);
  }
  sel.change(onResoChange);
}

function onFmtChange() {
  var sel = $('#sfmt');
  var val = sel.val();
  if (val === 'add') return;
  sel.val('add');
  var ary = getFormats();
  for (var i = 0; i < ary.length; i++) {
    if (JSON.stringify(ary[i]) === val) return;
  }
  try {
    val = JSON.parse(val);
    ary.push(val);
    setFormats({fmt: ary});
  }
  catch (e) {
    // ignore
  }
}

function initFormats() {
  var sel = $('#sfmt');
  for (var gname in formats) {
    var grp = $('<optgroup></optgroup>');
    grp.attr('label', gname);
    var gmap = formats[gname];
    for (var key in gmap) {
      var opt = $('<option></option>');
      var ary = gmap[key];
      opt.val(JSON.stringify(ary));
      opt.text(key);
      grp.append(opt);
    }
    sel.append(grp);
  }
  sel.change(onFmtChange);
}

function getFormats() {
  var ary = [];
  var fmt = $('#fmt').val().trim();
  if (fmt.length === 0) return ary;
  fmt = fmt.split('\n');
  var valid = true;
  for (var i = 0; i < fmt.length; i++) {
    var ent = fmt[i].trim();
    if (ent.length > 0) {
      try {
        ent = JSON.parse(ent);
        if (checkFormat(ent)) {
          ary.push(ent);
        }
        else {
          valid = false;
        }
      }
      catch (e) {
        valid = false;
      }
    }
  }
  if (!valid) {
    is.showError('Native media formats has invalid data.', 3000);
  }
  return ary;
}

function setFormats(cfg) {
  var ary = [];
  var fmt = cfg.fmt;
  for (var i = 0; i < fmt.length; i++) {
    ary.push(JSON.stringify(fmt[i]));
  }
  if (ary.length) {
    $('#fmt').val(ary.join(is.nl()));
  }
  else {
    $('#fmt').val('');
  }
}

function setReso(cfg) {
  if (!Array.isArray(cfg.sreso)) {
    return;
  }
  var sel = $('#sreso');
  var val = cfg.sreso[0] + 'x' + cfg.sreso[1];
  sel.val(val);
  if (sel.val() !== val) {
    sel.val('cst');
    $('#whrow').show();
  }
  else {
    $('#whrow').hide();
  }
}

function getReso() {
  var wh = $('#sreso').val();
  if (wh === 'cst') {
    var w = $('#w').val();
    var h = $('#h').val();
    w = parseInt(w, 10);
    h = parseInt(h, 10);
    if (isNaN(w) || w < 300) {
      is.showWarn('Invalid resolution width value', 3000);
      $('#w').focus();
      return null;
    }
    if (isNaN(h) || h < 200) {
      is.showWarn('Invalid resolution height value', 3000);
      $('#h').focus();
      return null;
    }
    return [w, h];
  }
  else {
    wh = wh.split('x');
    return [parseInt(wh[0], 10), parseInt(wh[1], 10)];
  }
}

function getArt() {
  var wh = $('#arts').val();
  wh = wh.split('x');
  return [parseInt(wh[0], 10), parseInt(wh[1], 10)];
}

function setArt(cfg) {
  var wh = cfg.arts;
  $('#arts').val(wh[0] + 'x' + wh[1]);
  $('#artp').val(cfg.artp);
}

function onResoChange() {
  var val = $('#sreso').val();
  var row = $('#whrow');
  var spd = 200;
  if (val === 'cst') {
    row.slideDown(spd, function() {
      $('#w').focus();
    })
  }
  else {
    row.slideUp(spd);
  }
}

function getMimeMap() {
  var mmap = {};
  var text = $('#mime').val().trim();
  if (text.length > 0) {
    var ary = text.split('\n');
    for (var i = 0; i < ary.length; i++) {
      var part = ary[i].trim();
      if (part.length > 0) {
        var kv = part.split('=');
        if (kv.length === 2) {
          var k = kv[0].trim();
          var v = kv[1].trim();
          if (k.length > 0 && v.length > 0) {
            mmap[k] = v;
          }
        }
      }
    }
  }
  return mmap;
}

function initMimeMap(mmap) {
  if (!mmap) return;
  var ary = [];
  for (var key in mmap) {
    ary.push(key + ' = ' + mmap[key]);
  }
  $('#mime').val(ary.join(is.nl()));
}

function setRadio(cls, val) {
  $(cls).each(function() {
    var rd = $(this);
    if (rd.val() == val) {
      rd.prop('checked', true);
      return false;
    }
  });
}

function getRadio(cls) {
  var rv = $(cls + ':checked').val();
  return parseInt(rv);
}

function readData() {
  var cfg = {};
  cfg.title = xcfg.title;
  var alias = $('#alias').val().trim();
  if (alias.length === 0) {
    alias = null;
  }
  cfg.alias = alias;
  cfg.sreso = getReso();
  if (!cfg.sreso) return null;
  cfg.acode = $('#acode').val();
  cfg.vcode = $('#vcode').val();
  cfg.tvs   = getRadio('.tvs');
  cfg.chn   = getRadio('.chn');
  cfg.asr   = parseInt($('#asr').val(), 10);
  cfg.tseek = parseInt($('#tseek').val(), 10);
  cfg.arts  = getArt();
  cfg.artp  = $('#artp').val();
  cfg.vtmb  = $('#vtmb').prop('checked');
  cfg.fmt   = getFormats();
  cfg.mime  = getMimeMap();
  cfg.mp2v  = getEncoder('mp2v');
  return cfg;
}

function fillData(cfg) {
  try {
    setReso(cfg);
    $('#acode').val(cfg.acode);
    $('#vcode').val(cfg.vcode);
    onTcodeChange();
    setRadio('.tvs', cfg.tvs);
    setRadio('.chn', cfg.chn);
    $('#asr').val(cfg.asr);
    $('#tseek').val(cfg.tseek);
    setArt(cfg);
    $('#vtmb').prop('checked', cfg.vtmb);
    setFormats(cfg);
    initMimeMap(cfg.mime);
    setEncoder('mp2v', cfg)
  } catch (e) {
    is.showError('Error setting data: ' + e, 3000);
  }
}

$(document).ready(function() {
  initReso();
  initFormats();
  fillData(xcfg)
});

function onTcodeChange() {
  var val = $('#vcode').val();
  $('#advbtn').enable((val !== 'none'));
}

//-----------------------------------------------------------------------------
// Import - Export
//-----------------------------------------------------------------------------

function impResize() {
  var wnd = $(window);
  var txt = $('#imp-text');
  txt.height(wnd.height() - 40 - 50 - 20);
}

$(document).ready(impResize);
$(window).resize(impResize);

function impCancel() {
  is.fadeShow('#imp', '#cfg');
}

function prepImport(mode) {
  $('#imp-hdr').text(mode ?
    is.tr('Export Device Configuration') :
    is.tr('Import Device Configuration'));
  $('#imp-hint').text(mode ?
    is.tr('Copy device configuration data to clipboard') :
    is.tr('Paste device configuration from clipboard and press Import'));
  if (mode) {
    var cfg = readData();
    if (!cfg) return;
    delete cfg.title;
    delete cfg.alias;
    delete cfg.sreso;
    $('#imp-btn').hide();
    try {
      $('#imp-text').prop('readonly', true).val(JSON.stringify(cfg, null, 0));
    }
    catch (e) {
      is.showError('Error preparing data: ' + e, 3000);
      return;
    }
  }
  else {
    $('#imp-btn').show();
    $('#imp-text').prop('readonly', false).val('');
  }
  is.fadeShow('#cfg', '#imp', function() {
    $('#imp-text').focus().select();
  });
}

function doImport() {
  prepImport(0);
}

function doExport() {
  prepImport(1);
}

function checkConfig(cfg) {
  if (!checkObj(cfg)) {
    throw "Import data is not an object";
  }
  if (!checkSelect(cfg.acode, 'none', 'lpcm-full', 'lpcm-short')) {
    throw "Invalid audio transcoding option";
  }
  if (!checkSelect(cfg.vcode, 'none', 'mp2v', 'x264')) {
    throw "Invalid video transcoding option";
  }
  if (!checkSelect(cfg.tvs, 0, 1)) {
    throw "Invalid TV system value";
  }
  if (!checkSelect(cfg.chn, 2, 6, 8)) {
    throw "Invalid number of audio channels";
  }
  if (!checkSelect(cfg.asr, 44100, 48000)) {
    throw "Invalid audio sample rate";
  }
  if (!checkWH(cfg.arts, 0, 800)) {
    throw "Invalid artwork resolution";
  }
  if (!checkSelect(cfg.artp, 'none', 'JPEG_TN')) {
    throw "Invalid artwork profile";
  }
  var fmt = cfg.fmt;
  if (!Array.isArray(fmt)) {
    throw "Native media formats not an array";
  }
  for (var i = 0; i < fmt.length; i++) {
    if (!checkFormat(fmt[i])) {
      throw "Invalid native media format at index: " + i;
    }
  }
  var mime = cfg.mime;
  if (mime === null || mime === undefined || typeof mime !== 'object') {
    throw "Mime mapping is not an object";
  }
  for (var key in mime) {
    var val = mime[key];
    if (typeof val !== 'string' || val.trim().length === 0) {
      throw "Invalid mime mapping entry";
    }
  }
  chkEncoder('mp2v', cfg.mp2v);
  return true;
}

//-----------------------------------------------------------------------------

function tryImport() {
  var txt = $('#imp-text').val().trim();
  if (txt.length === 0) {
    is.showError('Empty import data', 5000);
    return;
  }
  try {
    var cfg = JSON.parse(txt);
    checkConfig(cfg);
    fillData(cfg);
    impCancel();
    is.showInfo('Import finished. Save config to store changes.', 5000);
  } catch (ex) {
    is.showError(ex.toString(), 5000);
  }
}

//-----------------------------------------------------------------------------

function impFormat() {
  try {
    var imp = $('#imp-text');
    var txt = imp.val().trim();
    if (txt.length === 0) return;
    var obj = JSON.parse(txt);
    imp.val(JSON.stringify(obj, null, 2)).focus().select();
  } catch (ex) {
    is.showWarn(ex.toString(), 3000);
  }
}

//-----------------------------------------------------------------------------
// Advanced
//-----------------------------------------------------------------------------

function onAdvanced() {
  var aid = $('#vcode').val();
  is.fadeShow('#main', '#' + aid);
}

function showMime() {
  is.fadeShow('#main', '#mmap');
}

function onAdvBack(name) {
  is.fadeShow('#' + name, '#main');
}

//-----------------------------------------------------------------------------
// Encoder settings
//-----------------------------------------------------------------------------

function getEncoder(enc) {
  var x = {};
  x.pro = getVal(enc, 'pro');
  x.vbr = getInt(enc, 'vbr');
  x.mbr = getInt(enc, 'mbr');
  x.buf = getInt(enc, 'buf');
  x.inp = getStr(enc, 'inp');
  x.enc = getStr(enc, 'enc');
  x.ver = getVer(enc);
  return x;
}

function setEncoder(enc, cfg) {
  var def = DDEF[enc];
  var opt = cfg[enc] || def;
  setVal(enc, 'pro', opt.pro || def.pro);
  setVal(enc, 'vbr', opt.vbr || def.vbr);
  setVal(enc, 'mbr', opt.mbr);
  setVal(enc, 'buf', opt.buf || def.buf);
  setVal(enc, 'inp', opt.inp || def.inp);
  setVal(enc, 'enc', opt.enc || def.enc);
}

function chkEncoder(enc, opt) {
  if (!checkObj(opt)) {
    throw "Invalid encoder object in " + enc;
  }
  if (!checkStr(opt.pro, true)) {
    throw "Invalid video profile in " + enc;
  }
  if (!checkInt(opt.vbr, 1000, 20000)) {
    throw "Invalid video bitrate in " + enc;
  }
  if (!checkInt(opt.mbr, -1, 10000)) {
    throw "Invalid video max rate in " + enc;
  }
  if (!checkInt(opt.buf, 1, 128)) {
    throw "Invalid video buffer size in " + enc;
  }
  if (!checkStr(opt.inp)) {
    throw "Invalid input options in " + enc;
  }
  if (!checkStr(opt.enc)) {
    throw "Invalid encoder options in " + enc;
  }
}

//-----------------------------------------------------------------------------
// Value Helpers
//-----------------------------------------------------------------------------

function setVal(prefix, suffix, value) {
  $('#' + prefix + '-' + suffix).val(value);
}

function getVal(prefix, suffix) {
  return $('#' + prefix + '-' + suffix).val();
}

function getInt(prefix, suffix) {
  var val = parseInt(getVal(prefix, suffix), 10);
  return isNaN(val) ? 0 : val;
}

function getStr(prefix, suffix) {
  var val = getVal(prefix, suffix).trim();
  return (val.length > 0) ? val : null;
}

function getVer(enc) {
  if (DDEF) {
    var opt = DDEF[enc];
    if (opt) {
      return opt.ver || 0;
    }
  }
  return 0;
}

//-----------------------------------------------------------------------------
// Validation helpers
//-----------------------------------------------------------------------------

function checkWH(ary, min, max) {
  if (Array.isArray(ary) && ary.length === 2) {
    var w = ary[0], h = ary[1];
    if (typeof w === 'number' && typeof h === 'number') {
      if (w >= min && h >= min && w <= max && h <= max) {
        return true;
      }
    }
  }
  return false;
}

function checkSelect(val) {
  if (val !== null && val !== undefined) {
    var len = arguments.length;
    for (var i = 1; i < len; i++) {
      if (val === arguments[i]) {
        return true;
      }
    }
  }
  return false;
}

function checkInt(val, min, max) {
  return (typeof val === 'number' && val >= min && val <= max);
}

function checkObj(val) {
  return (val !== null && typeof val === 'object' && !Array.isArray(val));
}

function checkStr(val, req) {
  if (req) {
    return (typeof val === 'string' && val.trim().length > 0);
  }
  else {
    return (typeof val === 'string' || val === nu);
  }
}

function checkAryStr(ary, ix) {
  var val = ary[ix];
  if (typeof val === 'string') {
    val = val.trim();
    if (val.length > 0) {
      ary[ix] = val;
      return true;
    }
  }
  return false;
}

function checkArray(ary) {
  if (!Array.isArray(ary)) return false;
  var len = ary.length;
  for (var i = 0; i < len; i++) {
    if (!checkAryStr(ary, i)) return false;
  }
  return true;
}

function checkFormat(ary) {
  if (!Array.isArray(ary)) return false;
  var len = ary.length;
  if (len < 2 || len > 5) return false;
  var kind = ary[IX_KIND];
  if (kind !== 'a' && kind !== 'v') return false;
  if (!checkAryStr(ary, IX_FORMAT)) return false;
  switch (len) {
    case 2:
      return true;
    case 3:
      return checkArray(ary[IX_AUDIO]);
    case 4:
      return (checkArray(ary[IX_AUDIO]) && checkArray(ary[IX_VIDEO]));
    case 5:
      return (checkArray(ary[IX_AUDIO]) && checkArray(ary[IX_VIDEO]) && checkArray(ary[IX_PROFILE]));
  }
  return false;
}


//-----------------------------------------------------------------------------
// Reset
//-----------------------------------------------------------------------------

function onReset() {

  if (!confirm(is.tr('Are you sure you want to reset device configuration to default values') + '?')) {
    return;
  }

  fillData(DDEF);

}