/*
 * Copyright (c) 2015 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

'use strict';

var spawn  = require('child_process').spawn;
var byline = require('byline');

var env = require('../env');
var log = require('../log');

//-----------------------------------------------------------------------------

var GLOBAL_ARGS = [
  '-hide_banner',
  '-nostats',
  '-y'
];

//-----------------------------------------------------------------------------

function Coder(csn, opt) {

  var self  = this;
  var islog = log.canCoder();
  var ended = false;
  var res   = opt.res;

  //-----------------------------------------------------------------------------

  function onSocketClose() {
    resEnd();
  }

  function onResFinish() {
    ended = true;
    if (opt.ecb) {
      setImmediate(opt.ecb);
    }
    unbind();
  }

  function resEnd() {
    if (!ended) {
      ended = true;
      res.end();
    }
  }

  function bind() {
    res.once('finish', onResFinish);
    res.once('close', onSocketClose);
  }

  function unbind() {
    res.removeListener('finish', onResFinish);
    res.removeListener('close', onSocketClose);
  }

  //-----------------------------------------------------------------------------

  function launch(arg) {

    if (islog) {
      log.coder('[%s] %s', csn, arg.join(' '));
    }

    var all = GLOBAL_ARGS.concat('-loglevel', islog ? 'error' : 'fatal', arg);

    all.unshift(opt.bin);

    res.set('IsedoraStream', '1');

    res.end(JSON.stringify(all), 'utf8');

  }

  function start() {
    bind();
    opt.buildArgs(function(err, arg) {
      if (ended) {
        unbind();
        return;
      }
      launch(arg);
    });
  }

  //-----------------------------------------------------------------------------

  self.start = start;

}

exports.Coder = Coder;
