/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

// DLNA Flags

'use strict';

//var log = require('../log');
var bitsy = require('bitsy');

var PADDING = "000000000000000000000000";

var f = {
  /**
   * The sp-flag indicates if the Content Source will act as the Clock Source, for the context of the protocolInfo.
   */
  sender_paced:        0, //31,

  time_based_seek:     1, //30,
  byte_based_seek:     2, //29,

  /**
   * The playcontainer-param flag indicates support for a DLNA PlayContainer URI operation.
   * If the flag is true for a protocolInfo, then it means that the UPnP AV MediaRenderer can play that type of
   * content in a DLNA PlayContainer URI operation.
   */
  play_container:      3, //28,
  /**
   * If true, then the content does not have a fixed beginning.
   * Otherwise, the content does have a fixed beginning (i.e. npt=0 and byte-pos=0 map to the beginning).
   */
  s0_increasing:       4, //27,
  /**
   * If true, then the content does not have a fixed ending. Otherwise, the content has a fixed ending.
   */
  sn_increasing:       5, //26,

  rtsp_pause:          6, //25,
  /**
   * If the tm-s flag is true, then the associated Media Transport Content Source shall be capable of
   * supporting the Streaming Mode Transfer for the context of protocolInfo.
   */
  streaming_mode:      7, //24,
  /**
   * If the tm-i flag is true, then the associated Media Transport Content Source shall be capable of
   * supporting the Interactive Mode Transfer for the context of protocolInfo.
   * Image Media Class content shall set at least the tm-i flag equal to true.
   */
  interactive_mode:    8, //23,
  /**
   * If the tm-b flag is true, then the associated HTTP server shall be capable of supporting
   * the Background Mode Transfer for the context of the protocolInfo.
   */
  background_mode:     9, //22,
  /**
   * If the http-stalling flag is true, then the associated HTTP server shall be capable of supporting the
   * Connection Stalling method for the Pause and Pause-Release media operations on the content binary
   * and in addition the sp-flag shall be false.
   *
   */
  http_stalling:       10, //21,
  /**
   * If the dlna-v1.5-flag of the primary-flags token is true, then it shall mean the following.
   * Bits [31,21] and Bits [16,14] (inclusive) of the primary-flags token are valid for use (i.e. those bits are valid for use).
   * Bits [19,17] and Bits [13,0] of the primary-flags token have undefined values.
   */
  dlna_v15:            11 //20
};


//SENDER_PACED | S0_INCREASE | SN_INCREASE | INTERACTIVE_TRANSFERT_MODE |
//BACKGROUND_TRANSFERT_MODE | CONNECTION_STALL | DLNA_V15;
//exports.PHOTO = '8CF00000000000000000000000000000';

//SENDER_PACED | S0_INCREASE | SN_INCREASE | STREAMING_TRANSFER_MODE |
//BACKGROUND_TRANSFERT_MODE | CONNECTION_STALL | DLNA_V15;
//exports.MUSIC = '8D700000000000000000000000000000';

//SENDER_PACED | S0_INCREASE | SN_INCREASE | STREAMING_TRANSFER_MODE |
//BACKGROUND_TRANSFERT_MODE | CONNECTION_STALL | DLNA_V15;
//exports.VIDEO = '8D700000000000000000000000000000';

//-----------------------------------------------------------------------------

exports.PHOTO = bits(
  f.sender_paced, f.interactive_mode, f.background_mode, f.http_stalling, f.dlna_v15
);

exports.MUSIC = bits(
  f.sender_paced, f.s0_increasing, f.sn_increasing, f.streaming_mode, f.background_mode, f.http_stalling, f.dlna_v15
);

exports.VIDEO = bits(
  f.sender_paced, f.s0_increasing, f.sn_increasing, f.streaming_mode, f.background_mode, f.http_stalling, f.dlna_v15
);

//-----------------------------------------------------------------------------

exports.SBYTE = bits(
  f.sender_paced, f.byte_based_seek, f.streaming_mode, f.background_mode, f.http_stalling, f.dlna_v15
);

exports.STIME = bits(
  f.sender_paced, f.time_based_seek, f.streaming_mode, f.background_mode, f.http_stalling, f.dlna_v15
);

exports.SNONE = bits(
  f.sender_paced, f.s0_increasing, f.sn_increasing, f.streaming_mode, f.background_mode, f.http_stalling, f.dlna_v15
);

//-----------------------------------------------------------------------------

var CONVERSION = {
  none:       0,
  transcoded: 1
};

exports.ci = CONVERSION;

var OPERATION = {
  none:           '00',
  range:          '01',
  timeseek:       '10',
  range_timeseek: '11'
};

exports.op = OPERATION;

//-----------------------------------------------------------------------------

function bits() {
  var b = bitsy(32);
  for (var i = 0; i < arguments.length; i++) {
    b.set(arguments[i], true);
  }
  return b.toString().toUpperCase() + PADDING;
}
