/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

'use strict';

var fs  = require('fs');
var np  = require('path');
var log = require('../log');

var WOPTS = { persistent: false };

function FileWatcher(changecb) {

  var self  = this;
  var watch = null;
  var fpath = null;

  function stopWatch() {
    if (watch) {
      watch.removeAllListeners();
      try { watch.close() } catch (e) {}
      watch = null;
    }
  }

  function startWatch(filePath) {
    fpath = filePath;
    try {
      watch = fs.watch(fpath, WOPTS);
      watch.once('change', onChange);
      watch.once('error', onError);
      return true;
    } catch (ex) {
      log.error('FS watch error: %s (%s)', filePath, ex);
      return false;
    }
  }

  function onError(err) {
    stopWatch();
    changecb(err, self);
  }

  function onChange() {
    stopWatch();
    changecb(null, self);
  }

  self.start = function(filePath) {
    stopWatch();
    return startWatch(filePath);
  };

  self.restart = function() {
    if (fpath) {
      stopWatch();
      return startWatch(fpath);
    }
    return false;
  };

  self.fpath = function() { return fpath };

  self.dirname = function() { return (fpath) ? np.dirname(fpath) : null };

  self.stop  = function() {
    stopWatch();
    fpath = null;
  };

  self.isWatching = function() { return (watch !== null) };

}

exports.FileWatcher = FileWatcher;