/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

// Handler Requests

'use strict';

var rti = require('../rti');

var mtype = rti.mtype;

//-----------------------------------------------------------------------------

function BrowseRequest(dev, oid, req, meta, surl, start, count) {

  this.dev = dev;
  this.oid = oid;
  this.req = req;
  this.uid = null;
  this.ary = [];
  this.mt = 0;
  this.meta = meta;
  this.surl = surl;

  this.start = start;
  this.count = count;
  this.range = false;
  this.total = 0;

  this.dir   = {};
  this.image = {};
  this.audio = {};
  this.video = {};
  this.text  = {};
  this.other = {};

}

//-----------------------------------------------------------------------------

BrowseRequest.prototype.append = function(item) {
  this.ary.push(item);
};

BrowseRequest.prototype.addall = function(items) {
  var len = items.length;
  for (var i = 0; i < len; i++) {
    var item = items[i];
    if (item !== null) {
      this.ary.push(item);
    }
  }
};

BrowseRequest.prototype.prepend = function(item) {
  this.ary.unshift(item);
};

BrowseRequest.prototype.insert = function(index, item) {
  this.ary.splice(index, 0, item);
};

BrowseRequest.prototype.item = function(index) {
  return this.ary[index];
};

BrowseRequest.prototype.indexOfMediaClass = function(mediaClass) {
  var len = this.ary.length;
  for (var i = 0; i < len; i++) {
    var item = this.ary[i];
    if (item.getMediaClass() === mediaClass) {
      return i;
    }
  }
  return -1;
};

BrowseRequest.prototype.isImage = function() {
  return (this.mt === mtype.image);
};

BrowseRequest.prototype.isAudio = function() {
  return (this.mt === mtype.audio);
};

BrowseRequest.prototype.isVideo = function() {
  return (this.mt === mtype.video);
};

Object.defineProperty(BrowseRequest.prototype, 'length', {
  get: function() { return this.ary.length }
});

exports.BrowseRequest = BrowseRequest;

//-----------------------------------------------------------------------------
// Stream Request
//-----------------------------------------------------------------------------

function StreamRequest(dev, req, res, uid) {
  this.dev = dev;
  this.req = req;
  this.res = res;
  this.uid = uid;
  this.prf = null;
}

exports.StreamRequest = StreamRequest;