/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

'use strict';

var http    = require('http');
var util    = require("util");
var events  = require("events");

var MAX_PORT_ERRORS = 20;

//-----------------------------------------------------------------------------

function HTTPServer() {

  var self = this;

  events.EventEmitter.call(self);

  var srv = null;
  var port;
  var running = false;
  var errors  = 0;
  var sockets = [];

  function onServerConnection(sock) {
    sockets.push(sock);
    sock.on('close', function() {
      var ix = sockets.indexOf(sock);
      if (ix > -1) {
        sockets.splice(ix, 1);
      }
    });
  }

  function restartPort() {
    //srv.listen(++port, '0.0.0.0');
    srv.listen(0, '127.0.0.1');
  }

  function onServerError(err) {
    running = false;
    if ((err.code == 'EADDRINUSE') && (++errors < 20)) {
      setTimeout(restartPort, 50 * errors);
    }
    else {
      self.emit('error', err);
    }
  }

  function onServerListening() {
    var addr = srv.address();
    port = addr.port;
    errors = 0;
    running = true;
    self.emit('ready', addr);
  }

  function onServerClose() {
    running = false;
    self.emit('close');
  }

  function start(app, httpPort) {

    port = httpPort;

    srv = http.createServer(app);

    srv.on('listening', onServerListening);
//    srv.on('connection', onServerConnection);
    srv.on('close', onServerClose);
    srv.on('error', onServerError);

    srv.listen(0, '127.0.0.1');

  }

  function stop() {
    if (srv) {
      srv.close();
      srv = null;
    }
  }

  // Public Interface

  self.running = function() {return running};
  self.port    = function() {return port};

  self.start = start;
  self.stop  = stop;

}

util.inherits(HTTPServer, events.EventEmitter);

module.exports = HTTPServer;