/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

// Sort Routines

'use strict';

var evt = require('../evt');
var cfg = require('../cfg');
var log = require('../log');
var rti = require('../rti');
var didl = require('../cls/didl');

var Container = didl.Container;

var dirFirst;

//-----------------------------------------------------------------------------

function sortFolders(d1, d2) {
  if (dirFirst) {
    var isdir1 = (d1 instanceof Container && !d1.isMultiItem());
    var isdir2 = (d2 instanceof Container && !d2.isMultiItem());
    if (isdir1 && !isdir2) return -1;
    if (isdir2 && !isdir1) return 1;
  }
  return 0;
}

//-----------------------------------------------------------------------------

function sortByName(d1, d2) {

  var rv = sortFolders(d1, d2);
  if (rv !== 0) return rv;

  var t1 = d1.getTitle();
  var t2 = d2.getTitle();

  if (!t1) {
    log.error('Undefined title: %s', log.stringify(d1));
    return 0;
  }

  if (!t2) {
    log.error('Undefined title: %s', log.stringify(d2));
    return 0;
  }

  t1 = t1.toLowerCase();
  t2 = t2.toLowerCase();

  return t1.localeCompare(t2);

}

//-----------------------------------------------------------------------------

function sortByNewest(d1, d2) {

  var rv = sortFolders(d1, d2);
  if (rv !== 0) return rv;

  var m1 = d1.getDate() || 0;
  var m2 = d2.getDate() || 0;

  return (m2 - m1);

}

//-----------------------------------------------------------------------------

function sortByOldest(d1, d2) {

  var rv = sortFolders(d1, d2);
  if (rv !== 0) return rv;

  var m1 = d1.getDate() || 0;
  var m2 = d2.getDate() || 0;

  return (m1 - m2);

}

//-----------------------------------------------------------------------------

exports.getSortProc = function(sort) {
  switch (sort) {
    case rti.sort.name:   return sortByName;
    case rti.sort.newest: return sortByNewest;
    case rti.sort.oldest: return sortByOldest;
  }
  return sortByName;
};

//-----------------------------------------------------------------------------

function onConfig() {
  dirFirst = cfg.dirFirst();
}

evt.onConfigReadyChange(onConfig);