/*
 * Copyright (c) 2015 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

'use strict';

var execFile = require('child_process').execFile;

var env = require('../env');
var cfg = require('../cfg');
var log = require('../log');
var md5 = require('../md5');
var fop = require('../fop');
var mdb = require('./mdb');

var vtext = mdb.vtext;

var BUILD_TIMEOUT = 10 * 1000;

//-----------------------------------------------------------------------------

function getSrtPath(videoPath, trackIndex, cb) {

  var uid = md5(videoPath) + '_' + trackIndex;

  if (vtext.has(uid)) {
    return cb(null, vtext.filePath(uid));
  }

  vtext.set(uid, 0);

  var tmr = null;
  var tpath = vtext.filePath(uid);

  var args = ['-hide_banner', '-y', '-loglevel', 'error'];

  args.push('-threads', 4);
  args.push('-i', videoPath);
  args.push('-map', '0:s:' + trackIndex);
  args.push('-f', 'ass');
  args.push(tpath);

  var bin = cfg.getEncBin(env.coderPath);

  log.debug('Generating SUB for: %s [%d]', videoPath, trackIndex);

  var start = Date.now();

  execFile(bin, args, {env: env.coderEnv}, function(error, stdout, stderr) {

    if (tmr) {
      clearTimeout(tmr);
    }

    if (error) {
      if (typeof stderr === 'string') stderr = stderr.trim();
      log.error('Error SUB for: %s\n%s', videoPath, stderr);
      if (cb) cb(error);
      return;
    }

    log.debug('SUB generated in: %s sec', log.millisPeriod(start));

    fop.fileSize(tpath, function(err, size) {
      if (!err) vtext.set(uid, size);
    });

    if (cb) cb(null, tpath);

  });

  tmr = setTimeout(function() {
    tmr = null;
    var xcb = cb;
    cb = null;
    xcb(null, tpath);
  }, BUILD_TIMEOUT);

}

exports.getSrtPath = getSrtPath;