/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

// ------------------
// CONNECTION MANAGER
//-------------------

'use strict';

var npath = require('path');
var util  = require('util');

var evt = require("../evt");
var env = require("../env");
var cfg = require("../cfg");
var log = require("../log");
var rti = require('../rti');
var upnp = require('../cls/upnp');

var Service = require('../cls/service').Service;

var id = rti.did;

var SERVICE_TYPE = 'urn:schemas-upnp-org:service:ConnectionManager:1';
var SERVICE_ID   = 'urn:upnp-org:serviceId:ConnectionManager';
var SERVICE_SCPD = npath.join(global['ISEDORA_DIR'], 'pub', 'UPnP_AV_ConnectionManager_1.0.xml');

//-----------------------------------------------------------------------------
// Constructor
//-----------------------------------------------------------------------------

function ConnectionManager(devPath) {

  Service.call(this, devPath, SERVICE_TYPE, SERVICE_ID, SERVICE_SCPD);

  var self = this;

  var sourceInfo = [];

  evt.postDlnaSource(sourceInfo);

  self.event('SinkProtocolInfo', null);
  self.event('SourceProtocolInfo', sourceInfo.join(','));
  self.event('CurrentConnectionIDs', null);

  //-----------------------------------------------------------------------------

  self.GetProtocolInfo = function(req, arg, cb) {
    cb(null, {Source: sourceInfo.join(','), Sink: null});
  };

  //-----------------------------------------------------------------------------

  self.GetCurrentConnectionIDs = function(req, arg, cb) {
    cb(null, {ConnectionIDs: null});
  };

  //-----------------------------------------------------------------------------

  self.GetCurrentConnectionInfo = function(req, arg, cb) {
    cb(null, {"ProtocolInfo": null, "PeerConnectionManager": null, "PeerConnectionID": null, "Direction": null, "Status": null})
  };

  //-----------------------------------------------------------------------------

  self.PrepareForConnection = function(req, arg, cb) {
    cb(null, null);
  };

  //-----------------------------------------------------------------------------

  self.ConnectionComplete = function(req, arg, cb) {
    cb(null, null);
  }

}

util.inherits(ConnectionManager, Service);

//-----------------------------------------------------------------------------
// Event
//-----------------------------------------------------------------------------

evt.onRootDeviceInit(function(dev) {
  if (dev[id.deviceType] === upnp.DMS1) {
    dev.addService(new ConnectionManager(dev.path));
  }
});
