/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

'use strict';

var fs = require('fs');
var events = require('events');

var fwr = require('../cls/fwatcher');
var evt = require('../evt');
var log = require('../log');
var cfg = require('../cfg');

var EVT_CHANGE = 'change';

var wmap  = {};
var sysid = 1;
var emitter = new events.EventEmitter();

//-----------------------------------------------------------------------------

exports.getSysID = function() { return sysid };
exports.onChange = function(listener) { emitter.on(EVT_CHANGE, listener) };

//-----------------------------------------------------------------------------

function onWatcherChange(err, wtr) {

  if (err) {
    log.error('Error watching top folder: %s (%s)', wtr.fpath());
    delete wmap[wtr.fpath()];
    return;
  }

  log.trace('Top folder changed: %s', wtr.fpath());

  sysid++;

  emitter.emit(EVT_CHANGE, sysid);

  wtr.restart();

}

//-----------------------------------------------------------------------------

function deleteWatchers() {
  for (var path in wmap) {
    var wtr = wmap[path];
    wtr.stop();
  }
  wmap = {};
}

//-----------------------------------------------------------------------------

function createWatchers(roots) {
  for (var path in roots) {
    var wtr = wmap[path];
    if (!wtr) {
      wtr = new fwr.FileWatcher(onWatcherChange);
      if (wtr.start(path)) {
        wmap[path] = wtr;
      }
    }
  }
}

//-----------------------------------------------------------------------------

function onConfigReady() {
  deleteWatchers();
  if (cfg.enableImage()) createWatchers(cfg.imageRoots());
  if (cfg.enableAudio()) createWatchers(cfg.audioRoots());
  if (cfg.enableVideo()) createWatchers(cfg.videoRoots());
}

function onConfigChange() {
  sysid++;
  onConfigReady();
}

//evt.onConfigReady(onConfigReady);
//evt.onConfigChange(onConfigChange);
//evt.onExit(deleteWatchers);
