
/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

'use strict';

var np = require('path');

var ISEDORA_CODER_PATH = 'ISEDORA_CODER_PATH';
var ISEDORA_RESDIR     = 'ISEDORA_RESDIR';

//-----------------------------------------------------------------------------

var isMac = (process.platform === 'darwin');
var isWin = (!isMac && process.platform === 'win32');
var isIoj = (parseInt(process.versions.v8) > 3);

exports.isMac = function() { return isMac };
exports.isWin = function() { return isWin };
exports.isIoj = function() { return isIoj };

function ifMac(rvmac, rvwin) {
  return (isMac) ? rvmac : rvwin;
}

exports.ifMac = ifMac;

function ifIoj(rviojs, rvnode) {
  return (isIoj) ? rviojs : rvnode;
}

exports.ifIoj = ifIoj;

//-----------------------------------------------------------------------------

var procDir = np.dirname(process.execPath);

exports.procDir = procDir;

if (process.env[ISEDORA_RESDIR] !== undefined) {
  exports.resourceDir = process.env[ISEDORA_RESDIR];
}
else {
  exports.resourceDir = (isMac) ? np.join(procDir, '../Resources') : np.join(procDir, 'res');
}

exports.coderPath = np.join(procDir, ifMac('iSedoraFfmpeg', 'iSedoraFfmpeg.exe'));

exports.coderEnv = mkCoderEnv();

exports.nullDevice = ifMac('/dev/null', 'NUL');

function mkCoderEnv() {
  var cenv = {};
  var penv = process.env;
  for (var key in penv) {
    cenv[key] = penv[key];
  }
  cenv[ISEDORA_CODER_PATH] = exports.coderPath;
  return cenv;
}

//-----------------------------------------------------------------------------

exports.ERR = {
  AGENT_NOT_FOUND   : 100,
  HTTP_SERVER_START : 101
};

var ECODES = {
  ENOENT: 'File path not found'
};

exports.err = function(err) {

  if (typeof err === 'object' && err.code) {
    var rv = ECODES[err.code];
    if (rv) return rv;
  }

  return err.toString();

};

exports.isAborted = function(err) {
  return err.code == 'ECONNABORT';
};

exports.werr = function(text, cb) {
  if (typeof text === 'string') {
    process.stderr.write(text + "\n", 'utf8', cb);
  }
  else {
    cb();
  }
};

exports.fatal = function(err) {
  process.stderr.write(String(err) + "\n", 'utf8', function() {
    require('./evt').quit(-1);
  });
};

exports.safecb = function(cb) {
  return function() { if (cb) cb() };
};

exports.estack = function(err) {
  var stack = err['stack'];
  return (stack) ? stack : err;
};

exports.fileURL = function(fpath) {

  if (isWin) {
    fpath = fpath.replace(/\\/g, '/');
  }

  var furl = 'file://localhost';

  if (fpath.charAt(0) === '/') {
    furl += encodeURI(fpath);
  }
  else {
    furl += '/' + encodeURI(fpath);
  }

  return furl;

};