/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

/**
 * Network Interfaces Registry
 */

'use strict';

var os = require('os');

var evt = require('./evt');
var log = require('./log');
var cfg = require('./cfg');

var UPDATE_INTERVAL = 10 * 1000; // 10 seconds
var IPv4 = 'IPv4';

var nfaces = {};
var interval = null;

function copyFace(face) {
  return {
    name:  face.name,
    addr:  face.addr,
    issys: face.issys,
    local: face.local
  };
}

function updateFaces() {

  var on  = [];
  var off = [];

  var nf = os.networkInterfaces();

  var stamp = Date.now();

  for (var name in nf) {
    var fary = nf[name];
    fary.forEach(function(f) {
      if (f.family === IPv4) {
        var addr = f.address;
        var face = nfaces[addr];
        if (face) {
          face.stamp = stamp;
        }
        else {
          face = {
            name:  name,
            addr:  addr,
            stamp: stamp,
            issys: cfg.isSysNetFace(name),
            local: f.internal
          };
          nfaces[addr] = face;
          on.push(face);
        }
      }
    });
  }

  for (var addr in nfaces) {
    var face = nfaces[addr];
    if (face.stamp < stamp) {
      off.push(face);
    }
  }

  off.forEach(function(face) {
    delete nfaces[face.addr];
  });

  off.forEach(function(face) {
    if (!face.local && face.issys) {
      log.debug('Network interface off: %s', face.addr);
      evt.postNetFaceOff(copyFace(face));
    }
  });

  on.forEach(function(face) {
    if (!face.local && face.issys) {
      log.debug('Network interface on : %s', face.addr);
      evt.postNetFaceOn(copyFace(face));
    }
  });

}

//-----------------------------------------------------------------------------

evt.onConfigReady(function() {
  setImmediate(updateFaces);
  interval = setInterval(updateFaces, UPDATE_INTERVAL);
});

evt.onExit(function() {
  if (interval) {
    clearInterval(interval);
  }
});

exports.nface   = function(addr) { return nfaces[addr] };
exports.nfaces  = function() { return nfaces };
exports.allAddr = function() { return Object.keys(nfaces) };
exports.ifName  = function(addr) { var face = nfaces[addr]; return (face) ? face.name  : undefined; };
exports.ifLocal = function(addr) { var face = nfaces[addr]; return (face) ? face.local : undefined; };
exports.hasAddr = function(addr) { return (nfaces[addr]) ? true : false };
exports.getFace = function(addr) { var face = nfaces[addr]; return (face) ? copyFace(face) : null};
exports.ifAddr  = function(name) {
  for (var addr in nfaces) {
    var face = nfaces[addr];
    if (face.name === name) {
      return addr;
    }
  }
  return undefined;
};

exports.sysFaces = function() {
  var fary = [];
  for (var addr in nfaces) {
    var face = nfaces[addr];
    if (face.issys && !face.local) {
      fary.push(copyFace(face));
    }
  }
  return fary;
};