/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

'use strict';

var evt = require('../evt');
var cfg = require('../cfg');
var agt = require('../agt');
var log = require('../log');

var pk = {
  system: 0,
  always: 1,
  stream: 2
};

var TIMER_INTERVAL = 60 * 1000; // 1 min

var status = pk.system;
var count  = 0;
var timer  = null;
var state  = false;

//-----------------------------------------------------------------------------

function reset() {
  log.debug('Resetting power management');
  state = false;
  agt.pm_reset();
}

function deny() {
  log.debug('Deny system to sleep');
  state = true;
  agt.pm_deny();
}

function allow() {
  log.debug('Allow system to sleep');
  state = false;
  agt.pm_allow();
}

function allowIfNeeded() {
  if (count === 0 && state) {
    allow();
  }
  reschedule();
}

function stopTimer() {
  if (timer) {
    clearTimeout(timer);
    timer = null;
  }
}

function reschedule() {
  stopTimer();
  timer = setTimeout(allowIfNeeded, TIMER_INTERVAL);
}

//-----------------------------------------------------------------------------

function init(newStatus) {
  status = newStatus;
  reset();
  switch (status) {
    case pk.system:
      stopTimer();
      break;
    case pk.always:
      stopTimer();
      deny();
      break;
    case pk.stream:
      deny();
      reschedule();
      break;
  }
}

evt.onConfigReadyChange(function() {
  var newStatus = cfg.getPower();
  if (status !== newStatus) {
    init(newStatus);
  }
});

//-----------------------------------------------------------------------------

function onBrowseRequest() {
  if (!state) deny();
  reschedule();
}

evt.onBrowseRequest(function() {
  if (status === pk.stream) {
    setImmediate(onBrowseRequest);
  }
});

evt.onStartStream(function() {
  if (status === pk.stream && !state) {
    setImmediate(deny);
  }
  count++;
});

evt.onStopStream(function() {
  count--;
});

