/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

// Plugins Loader

'use strict';

var fs = require('fs');
var path = require('path');

var log = require('./log');
var evt = require('./evt');
var cfg = require('./cfg');

var plugins = {};
var packages = {};

exports.allPackages = function() { return packages };

exports.getPlugin = function(name) { return plugins[name] };
exports.getPackage = function(name) { return packages[name] };

//-----------------------------------------------------------------------------

function loadModules(mdir) {

  log.info('Loading modules from %s', mdir);

  var mpath;
  var mname;
  var ppath;

  var platform = process.platform;

  try {
    var names = fs.readdirSync(mdir);
    for (var i = 0; i < names.length; i++) {
      mname = names[i];
      mpath = path.join(mdir, mname);
      ppath = path.join(mpath, 'package.json');
      if (!fs.existsSync(ppath)) continue;
      log.debug('Loading module: %s', mname);
      try {
        var pkg = require(ppath);
        if (pkg.os && pkg.os.indexOf(platform) === -1) {
          log.debug('Skipping module: %s not compatible with %s', mname, platform);
          continue;
        }
        plugins[mname] = require(mpath);
        var pref = pkg.pref;
        if (pref) {
          pkg.pref = path.resolve(mpath, pref);
        }
        packages[mname] = pkg;
      } catch (e) {
        log.error('Module %s loading error: %s', mname, e);
      }
    }
    log.info('Modules loaded from %s', mdir);
  }
  catch (e) {
    log.error('Modules loading error: %s', e);
  }
}

//-----------------------------------------------------------------------------

function onConfigReady() {
  var mdir = path.join(global['ISEDORA_DIR'], 'plugins');
  loadModules(mdir);
}

evt.onConfigReady(onConfigReady);
