/*
 * Copyright (c) 2015 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

'use strict';

var np = require('path');
var spawn = require('child_process').spawn;
var byline = require('byline');

var env = require('./env');
var cfg = require('./cfg');
var log = require('./log');
var evt = require('./evt');
var sys = require('./sys');

var PPROC = np.join(env.procDir, env.ifMac('iSedoraProxy', 'iSedoraProxy.exe'));

function Proxy(localPort) {

  var self = this;

  var ready = false;
  var proc;
  var sout;
  var serr;
  var lerr;

  function _start(port) {

    var args = ['-p', port, '-l', localPort];

    log.debug('Starting proxy');

    proc = spawn(PPROC, args, {env: env.coderEnv, detached: false});

    proc.once('exit', onExit);
    proc.once('error', onError);

    proc.stdout.setEncoding('utf8');
    proc.stderr.setEncoding('utf8');

    sout = byline.createStream(proc.stdout);
    serr = byline.createStream(proc.stderr);

    sout.on('data', onStdOut);
    serr.on('data', onStdErr);

  }

  function start() {
    sys.checkport(cfg.httpPort(), function(err, port) {
      if (err) {
        env.fatal('Unable to find free server HTTP port');
        return;
      }
      _start(port);
    });
  }

  function onExit() {
    if (!evt.exiting()) {
      env.fatal(lerr || 'Unknown proxy error');
    }
  }

  function onError(err) {
    log.error('[PROXY] launch error: %s', err);
    env.fatal(err);
  }

  function onStdOut(line) {
    if (!ready) {
      ready = true;
      var port = parseInt(line, 10);
      if (isNaN(port)) {
        env.fatal('Inavlid proxy port: ' + line);
      }
      else {
        log.debug('Proxy ready on %s for %s', port, localPort);
        cfg.setHttpPort(port);
        evt.postHttpSrvReady({port: port});
      }
      return;
    }
    if (log.canCoder()) {
      log.coder('%s', line);
    }
  }

  function onStdErr(line) {
    lerr = line;
    log.debug('[PROXY] %s', line);
  }

  self.start = start;

}

exports.Proxy = Proxy;



