#!/usr/local/bin/node

var os = require('os');
var dgram = require('dgram');

var log = require('../lib/log');
var upnp = require('../lib/cls/upnp');

var mx = 1;
var st = process.argv[2];

if (!st || st === '1') st = upnp.ROOT_DEVICE;
if (st === '2') st = upnp.DMS1;

var faces = os.networkInterfaces();

for (var name in faces) {
  var ary = faces[name];
  for (var i = 0; i < ary.length; i++) {
    var addr = ary[i];
    if (addr['family'] === 'IPv4' && ! addr['internal']) {
      search(addr.address);
    }
  }
}


function search(addr) {

  log.info('Searching %s for %s', addr, st);

  var header = [
    'M-SEARCH * HTTP/1.1',
    'HOST: 239.255.255.250:1900',
    'MAN: "ssdp:discover"',
      'MX: ' + mx,
      'ST: ' + st,
    upnp.EOL
  ];

  var timeout = (mx + 2) * 1000;

  var packet = new Buffer(header.join(upnp.EOL), 'ascii');

  var sock = dgram.createSocket('udp4');

  sock.on('message', onMessage);
  sock.on('error', onError);
  sock.on('listening', onListening);

  sock.bind(0, addr);

  // Search Response

  function onMessage(data, from) {
    var text = data.toString('ascii');
    log.info('Search response from %s:%d\n%s', from.address, from.port, log.border(text));
  }

  function onError(err) {
    closeSocket();
    log.error('SSDP search socket error: %s (%s)', addr, err);
  }

  function onListening() {
    sock.setMulticastTTL(4);
    sock.setMulticastLoopback(true);
    setImmediate(sendPacket);
    setTimeout(closeSocket, timeout);
  }

  var sendCount = 0;

  function sendPacket() {
    try {
      sock.send(packet, 0, packet.length, upnp.PORT, upnp.HOST, function(err, bytes) {
        if (err) {
          log.error('SSDP search send error on: %s (%s)', addr, err);
          closeSocket();
        }
        else {
          if (sendCount++ < 3) {
            //setTimeout(sendPacket, 100);
          }
        }
      });
    } catch (e) {
      log.error('SSDP search listening at %s error: %s', face.addr, e);
      closeSocket();
    }
  }

  function closeSocket() {
    if (sock) {
      try { sock.close() } catch (e) {}
      sock = null;
    }
  }

}


function die(msg) {
  console.error.apply(null, arguments);
  process.exit(-1);
}